@@ -18,14 +18,15 @@ def normalize_size(text: str, size: int):
     try:  # can be html or not
         tree = etree.parse(io.StringIO(text), parser)
         # text could contain style attribute
-        ele = tree.xpath('//span[starts-with(@style,"font-size:")]')[0]
-        if ele is not None:
-            ele.attrib['style'] = f"font-size:{size}pt"
-            return etree.tostring(ele, encoding='unicode')
+        ele = tree.xpath('//span[starts-with(@style,"font-size:")]')
+        if ele is not None and len(ele):
+            ele[0].attrib['style'] = f"font-size:{size}pt"
+            return etree.tostring(ele[0], encoding='unicode')
     except etree.XMLSyntaxError as e:
         # assume simple html string no surrounding tags
         return f'<span style="font-size:{size}pt">{text}</span>'
 
+
 @attrs.define
 class Answer:  # pylint: disable=too-few-public-methods
     """canvas answer see for complete list of (valid) fields
@@ -40,7 +41,8 @@ TITLE_SIZE = 24
 QA_SIZE = 12
 
 # the patterns
-title_pattern = re.compile(r"^<font size=\"(?P<fontsize>\d+)\"><u>(?P<text>.*)</u></font>")
+title_pattern = \
+    re.compile(r"^<font size=\"(?P<fontsize>\d+)\"><u>(?P<text>.*)</u></font>")
 title_style_pattern = \
     re.compile(r"^<span style=\"font-size:(?P<fontsize>[\dpt]+)\"><u>(?P<text>.*)</u>")
 
@@ -50,22 +52,30 @@ quiz_name_style_pattern = re.compile(
     r"^<span style=\"font-size:(?P<fontsize>[\dpt]+)"
     r"(;text-transform:uppercase)?\"><b>(?P<text>.*)\s*</b></span>")
 # special match Sam
-page_ref_style_pattern = re.compile(r'(\(pp\.\s+[\d-]+)')
+page_ref_style_pattern =\
+    re.compile(r'(\(pp\.\s+[\d-]+)')
 
-q_pattern_fontsize = re.compile(r'^(?P<id>\d+)[).]\s+'
-                                r'(?P<prefix><font size="(?P<fontsize>\d+)">)(?P<text>.*</font>)')
-q_pattern = re.compile(r"^(?P<id>\d+)[).]\s+(?P<text>.*)")
+q_pattern_fontsize = \
+    re.compile(r'^(?P<id>\d+)[).]\s+'
+               r'(?P<prefix><font size="(?P<fontsize>\d+)">)(?P<text>.*</font>)')
+q_pattern = \
+    re.compile(r"^(?P<id>\d+)[).]\s+(?P<text>.*)")
 
 # '!' before the text of answer marks it as the right answer
 # idea: use [\d+]  for partially correct answer the sum must be FULL_SCORE
 a_ok_pattern_fontsize = re.compile(
-    r'^(?P<id>[a-d])\)\s+(?P<prefix><font size="(?P<fontsize>\d+)">.*)(?P<fullscore>!)(?P<text>.*</font>)')
-a_ok_pattern = re.compile(r"^(?P<id>[a-d])\)\s+(?P<prefix>.*)(?P<fullscore>!)(?P<text>.*)")
+    r'^(?P<id>[a-d])\)\s+(?P<prefix><font size="(?P<fontsize>\d+)">.*)'
+    r'(?P<fullscore>!)(?P<text>.*</font>)')
+a_ok_pattern = \
+    re.compile(r"^(?P<id>[a-d])\)\s+(?P<prefix>.*)(?P<fullscore>!)(?P<text>.*)")
 # match a-d then ')' then skip whitespace and all chars up to '!' after answer skip </font>
 
-a_wrong_pattern_fontsize = re.compile(r'^(?P<id>[a-d])\)\s+'
-                                      r'(?P<prefix><font size="(?P<fontsize>\d+)">)(?P<text>.*</font>)')
-a_wrong_pattern = re.compile(r"^(?P<id>[a-d])\)\s+(?P<text>.*)")
+a_wrong_pattern_fontsize = \
+    re.compile(r'^(?P<id>[a-d])\)\s+'
+               r'(?P<prefix><font size="(?P<fontsize>\d+)">)(?P<text>.*</font>)')
+a_wrong_pattern =\
+    re.compile(r"^(?P<id>[a-d])\)\s+(?P<text>.*)")
+
 
 @dataclass()
 class Rule:
@@ -74,6 +84,7 @@ class Rule:
     type: str
     normalized_size: int = QA_SIZE
 
+
 rules = [
     Rule(name='title', pattern=title_pattern, type='Title',
          normalized_size=TITLE_SIZE),
@@ -94,12 +105,18 @@ rules = [
 
 
 def parse(text: str, normalize_fontsize=False):
-    """ determine the type and parsed values of a string by matching and returning a
-    tuple (question number/answer char, score (if answer), text, type, fontsize)
-
-    type is one of (Question, Answer, Title, Pageref, Quizname)"""
-
-    # a list of dicts 'rules' with fields name, pattern
+    """
+    :param text : text to parse
+    :param normalize_fontsize: if True change fontsizes
+    :return determine the type and parsed values of a string by matching a ruleset and returning a
+    tuple:
+    - question number/answer: int/char,
+    - score :int (if answer),
+    - text: str,
+    - type: str. One of ('Question','Answer', 'Title, 'Pageref', 'Quizname') or 'Not recognized'
+    """
+    def isqa(rule):
+        return rule.type in ('Question', 'Answer')
 
     for rule in rules:
         match = rule.pattern.match(text)
@@ -112,10 +129,9 @@ def parse(text: str, normalize_fontsize=False):
             score = FULL_SCORE if 'fullscore' in match.groupdict() else 0
             prefix = match.group('prefix') if 'prefix' in match.groupdict() else ''
             text = prefix + match.group('text').strip()
-            text = normalize_size(text, rule.normalized_size) if normalize_fontsize else text
-            # fs_str = match.group('fontsize') if 'fontsize' in match.groupdict() else None
-            # fontsize = int(fs_str) if fs_str and fs_str.isdigit() else fs_str
-            return id_norm, score, text, rule.type #, fontsize
+            text = normalize_size(text, rule.normalized_size) if (normalize_fontsize
+                                                                  and is_qa(rule)) else text
+            return id_norm, score, text, rule.type
 
     return None, 0, "", 'Not recognized'
 
@@ -124,9 +140,12 @@ def parse_document_d2p(filename: str, check_num_questions: int, normalize_fontsi
     """
         :param  filename: filename of the Word docx to parse
         :param check_num_questions: number of questrions in a section
-        :return List[
-            Tuples[ quiz_names: str,questions: List[ question_name: str,
-                                                     List[ Answers: Tuple[name, weight]]]"""
+        :param normalize_fontsize: if True change fontsizes
+        :returns a list of Tuples[
+        - quiz_names: str
+        - questions: List[
+            - question_name: str,
+            - List[ Answers: list of Tuple[name:str, weight:int]]]"""
     #  from docx produce a text with minimal HTML formatting tags b,i, font size
     #  1) questiontitle
     #    a) wrong answer
@@ -195,7 +214,6 @@ def parse_document_d2p(filename: str, check_num_questions: int, normalize_fontsi
             assert tot_weight == FULL_SCORE, \
                 f"Check right/wrong marking and weights in Q '{questions[0]}'\n Ans {questions[1]}"
 
-    # if not_recognized:
     print('--- not recognized --' if not_recognized else '--- all lines were recognized ---')
     for line in not_recognized:
         print(line)
@@ -206,7 +224,10 @@ def parse_document_d2p(filename: str, check_num_questions: int, normalize_fontsi
 if __name__ == '__main__':
     os.chdir('../data')
     print(f"We are in folder {os.getcwd()}")
-    result = parse_document_d2p(r'version1.docx', check_num_questions=5)
+    result = parse_document_d2p(r'version1.docx',
+                                check_num_questions=5)
     pprint(result)
-    result = parse_document_d2p(r'version2.docx', check_num_questions=64,normalize_fontsize=True)
+    result = parse_document_d2p(r'version2.docx',
+                                check_num_questions=64,
+                                normalize_fontsize=True)
     pprint(result)
